import qualified Data.Map.Strict       as M
import           Data.Monoid           (mempty)
import           Data.Traversable      (for)
import           System.IO.Error       (isDoesNotExistError)
import           Test.Hspec
import           Test.Hspec.QuickCheck
import           Test.QuickCheck

import           Test.FileSystem.Fake


main :: IO ()
main = hspec $
  describe "Test.FileSystem.Fake" $ do
    describe "readPath and writePath" $
      prop "written files are read as written" $ \(NonEmpty pathAndContents) -> do
        let action =
              for pathAndContents $ \(path, contents) -> do
                writePath path (contents :: String)
                readPath path
        runFileSystemM mempty action
          `shouldBe` (M.fromList pathAndContents, Right $ map snd pathAndContents)

    describe "readPath" $
      prop "always throws a DoesNotExist error given empty file system" $ \path -> do
        let action = readPath path
            (afterRun, Left err) = runFileSystemM mempty action
        afterRun `shouldBe` (mempty :: M.Map FilePath String)
        err `shouldSatisfy` isDoesNotExistError
